#############################################################################
##
#W    checkbin.gi                 The ModIsomExt package          Leo Margolis
#W                                                                Tobias Moede
##


#############################################################################
##
#O MIPBinSplit( p, n, k, start, step, list )
##
##
InstallMethod( MIPBinSplit, [IsPosInt, IsPosInt, IsObject, IsPosInt, IsPosInt, IsList], function(p, n, k, start, step, list)
local grps, algs, tabs, bins, l, m, vals, todo, i, j, d, splits, stime;

    if not IsPrimeInt(p) then
        Error("<p> must be a prime integer");
    fi;

    if not (IsPosInt(k) or IsBool(k)) then
        Error("<k> must be a positive integer or a boolean value");
    fi;

	stime := Runtime();
    l := Length(list);
 
    # Check whether we have a list of integers or a list of groups
    if ForAll(list, IsInt) then
        grps := List(list, x -> SmallGroup(p^n, x));
    elif ForAll(list, IsGroup) then
        grps := list;
    else
        Error("<L> must be a list of integers or a list of groups");
    fi;

    algs := List(grps, x -> GroupRing(GF(p), x));
    tabs := [];

    for i in [1..l] do
        tabs[i] := TableOfRadQuotient(algs[i], start);
    od;

    # Init bins
    bins := [[1..l]];
    splits := [];

    Info(InfoModIsomExt, 1, "Refine bin");

    # Refine by weights
    vals := List( tabs, x -> x.pre.weights );
    bins := RefineBinsByVals( bins, vals );
    Info(InfoModIsomExt, 1, "  Weights yields bins ", bins);
    todo := Flat(bins);

    if Length(todo) = 0 then 
        return rec( bins := [], splits := [[0, list]], time := Runtime()-stime ); 
    fi;

    # Start up and refine by aut group
    vals := [];
    for i in [1..l] do
        if i in todo then 
            InitCanoForm(tabs[i]); 
            vals[i] := [tabs[i].auto.size, tabs[i].auto.partition];
        fi;
    od;
    bins := RefineBinsByVals( bins, vals );
    Info(InfoModIsomExt, 1, "  Layer 1 yields bins ", bins);
    todo := Flat(bins);

    if Length(todo) = 0 then 
        return rec( bins := [], splits := splits, time := Runtime()-stime ); 
    fi;

    # Get limit
    m := Maximum( List(tabs, x -> Maximum(x.pre.weights)));
    if not IsBool(k) then m := Minimum(m,k); fi;

    # Loop and refine by canonical forms
    for j in [2..m] do
        if j > start then
	        start := start+step;
	        for i in todo do
	            WordFillTable(tabs[i], start);
			    TableOfRadByCollection(tabs[i]);
	        od;
	    fi;	

        vals := [];
    
        for i in [1..l] do
            if i in todo then 
                ExtendCanoForm(tabs[i], j);
				# Either use CompleteTable or do as in CompareTables 
				CompleteTable(tabs[i].cano);
                vals[i] := tabs[i].cano.tab;            
            else
	            tabs[i] := "done"; 
	        fi;
        od;
    
        bins := RefineBinsByVals( bins, vals );
        Info(InfoModIsomExt, 1, "  Layer ", j, " yields bins ", bins);
        
        # Record split information  
        if Length(Flat(bins)) < Length(todo) then    
            Add(splits, [j, list{Difference([1..l], Flat(bins))}]);  
        fi;
        todo := Flat(bins);
    
        if Length(todo) = 0 then 
            return rec( bins := [], splits := splits, time := Runtime()-stime );
        fi;
    od;

    return rec( bins := List(bins, x -> List(x, y -> list[y])), splits := splits, time := Runtime()-stime );
end );
